@@ -1,5 +1,5 @@
 /*
- *  Copyright 1999-2005 Vizdom Software, Inc. All Rights Reserved.
+ *  Copyright 1999-2006 Vizdom Software, Inc. All Rights Reserved.
  * 
  *  This program is free software; you can redistribute it and/or 
  *  modify it under the same terms as the Perl Kit, namely, under 
@@ -39,7 +39,7 @@ import com.vizdom.util.Debug;
  * only types defined within that module can be read.
  *
  * @author John Lacey
- * @version $Revision: 1.27 $
+ * @version $Revision: 1.29 $
  */
 public class BerModule
 {
@@ -64,6 +64,7 @@ public class BerModule
         protected void mWriteContents(OutputStream anOut) {}
         protected void mReadContents(InputStream anIn, 
             BerModule aModule, BerIdentifier anIdentifier, int aLength) {
+            // FIXME: The tests never execute this assertion.
             Debug.assertTrue(aLength == 0); }
         public String toString() { return "end-of-contents"; }
     };
@@ -108,14 +109,19 @@ public class BerModule
      * 
      * @param aFactory a BER object factory
      * @param anIdentifier a BER identifier
-     * @throws AssertionFailedException if the type is already registered
+     * @throws IllegalStateException if the type is already registered
      */
     public void registerFactory(BerObjectFactory aFactory,
         BerIdentifier anIdentifier)
     {
         Object previous = mTiedFactories.put(anIdentifier, aFactory);
-        if (Debug.ASSERT)
-            Debug.assertTrue(previous == null);
+        if (previous != null)
+        {
+            // ??? BerException?
+            throw new IllegalStateException(
+                "Type " + anIdentifier + " is already registered with " +
+                previous);
+        }
     }
 
 
