@@ -1,5 +1,5 @@
 /*
- * Copyright 1999-2001 Vizdom Software, Inc. All Rights Reserved.
+ * Copyright 1999-2002,2004-2006 Vizdom Software, Inc. All Rights Reserved.
  * 
  *  This program is free software; you can redistribute it and/or 
  *  modify it under the same terms as the Perl Kit, namely, under 
@@ -23,7 +23,6 @@ package com.vizdom.dbd.jdbc;
 import com.vizdom.ber.BerObject;
 import com.vizdom.ber.BerIdentifier;
 import com.vizdom.ber.BerTypes;
-import com.vizdom.util.Debug;
 import com.vizdom.util.UnreachableCodeException;
 import java.io.*;
 import java.lang.reflect.InvocationTargetException;
@@ -34,14 +33,16 @@ import java.net.Socket;
 import java.sql.*;
 import java.util.Hashtable;
 import java.util.Properties;
+import org.apache.log4j.Logger;
+import org.apache.log4j.NDC;
+
 
 /**
  * This class implements a single DBD server connection. It's 
  * intended to be run as a single thread started by a
  * connection-accepting server.
  *
- * @author Gennis Emerson
- * @version $Revision: 1.35 $
+ * @version $Revision: 1.38 $
  */
 /* The handleMessage methods may throw only SQLExceptions and 
  * RuntimeExceptions, the first if they can recover, the second
@@ -49,39 +50,52 @@ import java.util.Properties;
  */
 class Connection implements Runnable
 {
+    /** The log4j logger. */
+    private static final Logger gLog = Logger.getLogger(Connection.class);
+
     /** The DBI constant for non-nullable columns. */
     private static final Integer sDbiNoNulls = new Integer(0);
+
     /** The DBI constant for nullable columns. */
     private static final Integer sDbiNullable = new Integer(1);   
+
     /** The DBI constant for nullable-unknown columns. */
     private static final Integer sDbiNullableUnknown = new Integer(2);
+
     /** LONG fields will be read in chunks this size. */
     private static final int sLONG_READ_BUFFER_SIZE = 8192;
 
     /** The current thread name, used in tracing messages. */
     private String mThreadId;
+
     /** The client socket. */
     private Socket mSocket;
+
     /**
      * The client socket's input stream. This is a BufferedInputStream 
      * in order to avoid a problem I had with <code>available</code> 
      * being > 0 but <code>read</code> returning -1.
      */
     private BufferedInputStream mIn;
+
     /** The client socket's output stream. */
     private BufferedOutputStream mOut;
 
     /** This connection's BerModule. */
     private BerDbdModule mBerModule;
+
     /** The JDBC Connection. */
     private java.sql.Connection mConn;
+
     /** 
      * True if this Connection was constructed with a pre-existing
      * JDBC connection.
      */
     private boolean preExistingConnection;
+
     /** A collection of the Statements currently in use. */
     private Hashtable mStatementTable;
+
     /** 
      * The count of Statements created. This value is used as the
      * key into the statement table for each newly-created Statement.
@@ -89,6 +103,7 @@ class Connection implements Runnable
     /* Starts at 1 in case we want the statement handle to be true in perl. */
     private int mNextHandle;
 
+
     /**
      * Constructor - initializes fields.
      *
@@ -106,12 +121,11 @@ class Connection implements Runnable
         mOut = new BufferedOutputStream(mSocket.getOutputStream());
         mStatementTable = new Hashtable();
         mNextHandle = 1;
-
         mBerModule = aBerModule;
-
         preExistingConnection = false;
     }
 
+
     /**
      * Constructor. Uses a pre-existing JDBC connection.
      *
@@ -139,14 +153,15 @@ class Connection implements Runnable
      */
     public void run()
     {
-        mThreadId = "[" + Thread.currentThread().getName() + "] ";
+        mThreadId = "[" + Thread.currentThread().getName() + "]";
+
+        NDC.push(mThreadId); 
 
         BerObject request;
         BerObject response = null;
         boolean connected = true;
 
-        Debug.getLogWriter(Debug.BRIEF).println(mThreadId + 
-            "client started");
+        gLog.info("Client started");
 
         while (connected)
         {
@@ -160,14 +175,14 @@ class Connection implements Runnable
                  */
                 request = mBerModule.readFrom(mIn);
                 if (request == null)
-                 {
                     throw new FatalException("Client disconnected");
-                }
-                Debug.getLogWriter(Debug.BRIEF).println(mThreadId + 
-                    "request: " + request);
+                if (gLog.isDebugEnabled())
+                    gLog.debug("Request: " + request);
 
                 BerIdentifier id = request.getIdentifier(); 
-                Debug.assertTrue(id.getTagClass() == BerTypes.APPLICATION);
+                if (id.getTagClass() != BerTypes.APPLICATION)
+                    throw new FatalException("Unknown request received");
+
                 int tagNumber = id.getTagNumber();
                 switch (tagNumber)
                 {
@@ -250,8 +265,8 @@ class Connection implements Runnable
                 {
                     response.writeTo(mOut);
                     mOut.flush(); 
-                    Debug.getLogWriter(Debug.BRIEF).println(mThreadId + 
-                        "response: " + response);
+                    if (gLog.isDebugEnabled())
+                        gLog.debug("Response: " + response);
                     response = null;
                 }
                 else
@@ -259,24 +274,20 @@ class Connection implements Runnable
             }
             catch (SQLException sqlError)
             {
+                gLog.warn("Error", sqlError);
                 try
                 {
                     mSendError(sqlError);
                 }
                 catch (FatalException fatal)
                 {
-                    Debug.getLogWriter(Debug.BRIEF).print(mThreadId + 
-                        "error: ");
-                    fatal.printStackTrace(Debug.getLogWriter(Debug.BRIEF));
+                    gLog.warn("Failed to send error", fatal);
                 }
-                Debug.getLogWriter(Debug.BRIEF).print(mThreadId + "error: ");
-                sqlError.printStackTrace(Debug.getLogWriter(Debug.BRIEF));
             }
             catch (Throwable throwable)
             {
                 connected = false;
-                Debug.getLogWriter(Debug.BRIEF).print(mThreadId + "error: ");
-                throwable.printStackTrace(Debug.getLogWriter(Debug.BRIEF));
+                gLog.fatal("Error; ending connection", throwable);
                 try
                 {
                     mSendError(new DbdException(
@@ -285,19 +296,21 @@ class Connection implements Runnable
                 }
                 catch (FatalException fatal)
                 {
-                    Debug.getLogWriter(Debug.BRIEF).print(mThreadId + 
-                        "error: ");
-                    fatal.printStackTrace(Debug.getLogWriter(Debug.BRIEF));
+                    gLog.warn("Failed to send error", fatal);
                 }
             }
         }
         
+        mDoDisconnect(true); 
         try { mOut.close(); } catch (IOException e) { }
+        mOut = null; 
         try { mIn.close(); } catch (IOException e) { }
+        mIn = null; 
         try { mSocket.close(); } catch (IOException e) { }
-
-        Debug.getLogWriter(Debug.BRIEF).println(mThreadId + 
-            "client done");
+        mSocket = null;
+        
+        gLog.info("Client done");
+        NDC.pop();
     }
 
     /**
@@ -333,8 +346,8 @@ class Connection implements Runnable
             charset = aRequest.getCharacterEncoding();
             if (!charset.equals(""))
             {
-                Debug.getLogWriter(Debug.VERBOSE).println(mThreadId + 
-                    "setting character encoding to " + charset);
+                if (gLog.isDebugEnabled())
+                    gLog.debug("Setting character encoding to " + charset);
                 // Trigger UnsupportedEncodingException as soon as possible.
                 com.vizdom.util.CharacterEncoder.toByteArray("test string", 
                     charset);
@@ -348,18 +361,15 @@ class Connection implements Runnable
                 String user = aRequest.getUser(charset);
                 String password = aRequest.getPassword(charset);
                 Properties connProperties = aRequest.getProperties(charset);
-                if (Debug.getVerbosity() >= Debug.VERBOSE)
+                if (gLog.isDebugEnabled())
                 {
-                    Debug.getLogWriter(Debug.VERBOSE).println(mThreadId + 
-                        "url = " + aRequest.getURL(charset));
-                    Debug.getLogWriter(Debug.VERBOSE).println(mThreadId +  
-                        "user = " + aRequest.getUser(charset));
+                    gLog.debug("url = " + aRequest.getURL(charset));
+                    gLog.debug("user = " + aRequest.getUser(charset));
                     java.util.Enumeration keys = connProperties.keys();
                     while (keys.hasMoreElements())
                     {
                         String key = (String) keys.nextElement();
-                        Debug.getLogWriter(Debug.VERBOSE).println(mThreadId + 
-                            key + " = " + connProperties.get(key));
+                        gLog.debug(key + " = " + connProperties.get(key));
                     }
                 }
                 if (user == null && password == null)
@@ -393,23 +403,46 @@ class Connection implements Runnable
     BerObject handleRequest(DisconnectRequest aRequest)
         throws SQLException
     {
-        if (!preExistingConnection)
-        {
-            java.util.Enumeration elements = mStatementTable.elements();
-            while (elements.hasMoreElements())
-            {
-                StatementHolder holder = (StatementHolder) elements.nextElement();
-                holder.close();
-            }
-            Debug.getLogWriter(Debug.VERBOSE).println(mThreadId + 
-                "closed " + mStatementTable.size() + " Statements on disconnect");
-            
-            mStatementTable = null;
-            mConn.close();
-        }
+        mDoDisconnect(!preExistingConnection); 
         return new DisconnectResponse();
     }
     
+    /**
+     * Closes statements and the JDBC connection.
+     *
+     * @param shouldCloseConnection true if the connection should
+     * be closed. If a pre-existing connection was passed in, the
+     * connection should not be closed.
+     */
+    private void mDoDisconnect(boolean shouldCloseConnection)
+    {
+        if (mStatementTable != null)
+        {
+            java.util.Enumeration elements = mStatementTable.elements();
+            while (elements.hasMoreElements())
+                ((StatementHolder) elements.nextElement()).close();
+            if (gLog.isDebugEnabled())
+            {
+                gLog.debug("Closed " + mStatementTable.size() + 
+                    " Statements on disconnect");
+            }
+            mStatementTable = null;
+        }
+        if (mConn != null && shouldCloseConnection)
+        {
+            try
+            { 
+                gLog.debug("Closing database connection"); 
+                mConn.close();
+            } 
+            catch (Throwable t)
+            {
+                gLog.warn("Error closing connection: " + t.getMessage());
+            }
+        }
+        mConn = null; 
+    }
+
     /**
      * Checks to see whether the JDBC Connection is still open.
      *
@@ -469,8 +502,8 @@ class Connection implements Runnable
         int stmtHandle = mNextHandle++;
         mStatementTable.put(new Integer(stmtHandle), 
             new StatementHolder(stmt));
-        Debug.getLogWriter(Debug.VERBOSE).println(mThreadId + 
-            "assigned handle " + stmtHandle);
+        if (gLog.isTraceEnabled())
+            gLog.trace("Assigned statement handle " + stmtHandle);
         return new PrepareResponse(stmtHandle);
     }
     
@@ -497,27 +530,29 @@ class Connection implements Runnable
     BerObject handleRequest(ExecuteRequest aRequest)
         throws SQLException
     {
-        Debug.getLogWriter(Debug.VERBOSE).println(mThreadId + 
-            "executing statement handle " + aRequest.getHandle());
+        if (gLog.isTraceEnabled())
+            gLog.trace("Executing statement handle " + aRequest.getHandle());
         StatementHolder holder = mGetStatementHolder(aRequest.getHandle());
         PreparedStatement stmt = holder.getStatement();
         Parameter[] params = aRequest.getParameters();
-        Debug.getLogWriter(Debug.VERBOSE).println(mThreadId + 
-            "setting " + params.length + " parameters");
+        if (gLog.isDebugEnabled())
+            gLog.debug("setting " + params.length + " parameters");
         for (int i = 0; i < params.length; i++)
         {
             try 
             {
                 if (params[i].value == null)
                 {
-                    Debug.getLogWriter(Debug.TEDIOUS).println(mThreadId + 
-                        "setting parameter " + (i + 1) + "; value null; type " +
-                        params[i].type);
+                    if (gLog.isTraceEnabled())
+                    {
+                        gLog.trace("setting parameter " + (i + 1) + 
+                            "; value null; type " + params[i].type);
+                    }
                     stmt.setNull(i + 1, params[i].type);
                     continue;
                 }
-                Debug.getLogWriter(Debug.TEDIOUS).print(mThreadId + 
-                    "setting parameter " + (i + 1) + "; value ");
+                if (gLog.isTraceEnabled())
+                    gLog.trace("setting parameter " + (i + 1) + "; value ");
                 switch (params[i].type) 
                 {
                 case Types.BINARY:
@@ -581,16 +616,20 @@ class Connection implements Runnable
                     params[i].type == Types.VARBINARY ||
                     params[i].type == Types.LONGVARBINARY)
                 {
-                    Debug.getLogWriter(Debug.TEDIOUS).println(mThreadId + 
-                        "(binary; length " + 
-                        params[i].value.toByteArray().length + 
-                        "); type " + params[i].type);
+                    if (gLog.isTraceEnabled())
+                    {
+                        gLog.trace("(binary; length " + 
+                            params[i].value.toByteArray().length + 
+                            "); type " + params[i].type);
+                    }
                 }
                 else
                 {
-                    Debug.getLogWriter(Debug.TEDIOUS).println(mThreadId + 
-                        params[i].value.toString() + "; type " + 
-                        params[i].type);
+                    if (gLog.isTraceEnabled())
+                    {
+                        gLog.trace(params[i].value.toString() + "; type " + 
+                            params[i].type);
+                    }
                 }
             }
             catch (NumberFormatException ne)
@@ -612,8 +651,7 @@ class Connection implements Runnable
         if (stmt.execute())
         {
             // execute returned a result set.
-            Debug.getLogWriter(Debug.TEDIOUS).println(mThreadId + 
-                "getting result set");
+            gLog.debug("Getting and returning a result set");
             ResultSet rs = stmt.getResultSet();
             holder.setResultSet(rs);
             ResultSetMetaData rsmd = holder.getResultSetMetaData();
@@ -625,6 +663,7 @@ class Connection implements Runnable
         else
         {
             // execute returned a row count.
+            gLog.debug("Getting and returning a row count");
             resp = new ExecuteResponse(new ExecuteRowsResponse(
                 stmt.getUpdateCount()));
         }
@@ -652,8 +691,11 @@ class Connection implements Runnable
     BerObject handleRequest(FetchRequest aRequest)
         throws SQLException
     {
-        Debug.getLogWriter(Debug.VERBOSE).println(mThreadId + 
-            "fetching row from statement handle " + aRequest.getHandle());
+        if (gLog.isTraceEnabled())
+        {
+            gLog.trace("Fetching row from statement handle " + 
+                aRequest.getHandle());
+        }
         StatementHolder holder = mGetStatementHolder(aRequest.getHandle());
         ResultSet rs = holder.getResultSet();
         if (rs == null)
@@ -672,14 +714,18 @@ class Connection implements Runnable
                 holder.getProperties().get("LongTruncOk")).booleanValue();
             boolean chopBlanks = ((Boolean) 
                 holder.getProperties().get("ChopBlanks")).booleanValue();
+            boolean readAll = ((Boolean) 
+                holder.getProperties().get("jdbc_longreadall")).booleanValue();
             for (int i = 0; i < cols; i++)
             {
                 try
                 {
                     int type = rsmd.getColumnType(i + 1);
-                    Debug.getLogWriter(Debug.TEDIOUS).println(mThreadId + 
-                        "getting column " + (i + 1) + "/" + 
-                        rsmd.getColumnName(i + 1) + "; type " + type);
+                    if (gLog.isTraceEnabled())
+                    {
+                        gLog.trace("getting column " + (i + 1) + "/" + 
+                            rsmd.getColumnName(i + 1) + "; type " + type);
+                    }
                     switch (type)
                     {
                     case Types.BINARY: 
@@ -692,8 +738,9 @@ class Connection implements Runnable
                             row[i] = null;
                         else
                         {
-                            row[i] = mReadLong(i + 1, rs.getBinaryStream(i + 1),
-                                longReadLen, longTruncOk);
+                            row[i] = mReadLong(i + 1, 
+                                rs.getBinaryStream(i + 1),
+                                longReadLen, longTruncOk, readAll);
                         }
                         break;
 
@@ -709,7 +756,7 @@ class Connection implements Runnable
                             {
                                 row[i] = mReadLong(i + 1, 
                                     blob.getBinaryStream(), longReadLen, 
-                                    longTruncOk);
+                                    longTruncOk, readAll);
                             }
                         }
                         break;
@@ -728,7 +775,7 @@ class Connection implements Runnable
                         {
                             char[] chars = mReadLong(i + 1, 
                                 rs.getCharacterStream(i + 1), longReadLen, 
-                                longTruncOk);
+                                longTruncOk, readAll);
                             row[i] = (chars == null) ? 
                                 null : new String(chars); 
                         }
@@ -746,7 +793,7 @@ class Connection implements Runnable
                             {
                                 char[] chars = mReadLong(i + 1, 
                                     clob.getCharacterStream(), longReadLen, 
-                                    longTruncOk);
+                                    longTruncOk, readAll);
                                 row[i] = (chars == null) ? 
                                     null : new String(chars);
                             }
@@ -779,7 +826,7 @@ class Connection implements Runnable
                 {
                     throw new DbdException(DbdException.gFETCH_EXCEPTION,
                         new String[] { String.valueOf(i + 1), 
-                        ioError.toString() });
+                                       ioError.toString() });
                 }
             }
         }
@@ -871,10 +918,11 @@ class Connection implements Runnable
                 if (rs == null)
                     throw new DbdException(DbdException.gNO_CURSOR);
                 String[] cursorname = new String[1];
-                // DBI specifies that the cursor name should be returned as undef
-                // if cursor names are not supported. JDBC specifies that
-                // getCursorName should throw an exception if named cursors
-                // are not supported.
+                // DBI specifies that the cursor name should be
+                // returned as undef if cursor names are not
+                // supported. JDBC specifies that getCursorName
+                // should throw an exception if named cursors are
+                // not supported.
                 try
                 {
                     cursorname[0] = rs.getCursorName();
@@ -882,8 +930,7 @@ class Connection implements Runnable
                 catch (SQLException e)
                 {
                     cursorname[0] = null;
-                    Debug.getLogWriter(Debug.TEDIOUS).println(mThreadId + 
-                        "getCursorName threw an exception");
+                    gLog.warn("getCursorName threw an exception", e);
                 }
                 return new GetStatementPropertyResponse(cursorname,
                     mBerModule.getCharacterEncoding());
@@ -955,8 +1002,7 @@ class Connection implements Runnable
                         break;
                     default:  // UnreachableCodeException???
                         data[i - 1] = null;
-                        Debug.getLogWriter(Debug.TEDIOUS).println(mThreadId +
-                            "isNullable returned an unknown value " + 
+                        gLog.warn("isNullable returned an unknown value " + 
                             nullable);
                         break;
                     }
@@ -1007,6 +1053,12 @@ class Connection implements Runnable
                 new Boolean(aRequest.getPropertyValue().equals("1")));
             return new SetStatementPropertyResponse();
         }
+        if (property.equals("jdbc_longreadall"))
+        {
+            holder.getProperties().put(property, 
+                new Boolean(aRequest.getPropertyValue().equals("1")));
+            return new SetStatementPropertyResponse();
+        }
 
         throw new DbdException(DbdException.gUNKNOWN_PROPERTY, 
             new String[] { property });
@@ -1051,11 +1103,6 @@ class Connection implements Runnable
     BerObject handleRequest(StatementDestroyRequest aRequest)
         throws DbdException
     {
-        /*
-        if (true)
-            throw new DbdException(DbdException.gGENERIC_EXCEPTION,
-                new String[] { "Fake error in server statement destroy" });
-        */
         // Note: we want to call mStatementTable.remove, so don't use
         // mGetStatementHolder.
         int handle = aRequest.getHandle();
@@ -1066,8 +1113,8 @@ class Connection implements Runnable
             throw new DbdException(DbdException.gINVALID_STATEMENT_HANDLE);
         else
         {
-            Debug.getLogWriter(Debug.VERBOSE).println(mThreadId +
-                "destroying statement " + handle);
+            if (gLog.isTraceEnabled())
+                gLog.trace("Destroying statement " + handle);
             
             // Close the statement and result set. This may cause
             // a commit or rollback in the JDBC driver in
@@ -1094,12 +1141,12 @@ class Connection implements Runnable
         throws SQLException,DbdException
     {
         String methodName = aRequest.getMethodName();
-        Debug.getLogWriter(Debug.VERBOSE).println(mThreadId +
-            "func method: " + methodName);
+        if (gLog.isTraceEnabled())
+            gLog.trace("Func method: " + methodName);
         String value =  mHandleFunc(mConn, aRequest.getMethodName(), 
             aRequest.getParameters());
-        Debug.getLogWriter(Debug.VERBOSE).println(mThreadId +
-            methodName + " returned '" + value + "'");
+        if (gLog.isDebugEnabled())
+            gLog.debug(methodName + " returned '" + value + "'");
         try
         {
             return new ConnectionFuncResponse(value, 
@@ -1131,8 +1178,8 @@ class Connection implements Runnable
         throws SQLException,DbdException
     {
         String methodName = aRequest.getMethodName();
-        Debug.getLogWriter(Debug.VERBOSE).println(mThreadId +
-            "func method: " +  methodName);
+        if (gLog.isTraceEnabled())
+            gLog.trace("Statement func method: " +  methodName);
 
         int dot = methodName.lastIndexOf(".");
         if (dot == -1)
@@ -1143,8 +1190,8 @@ class Connection implements Runnable
             throw new DbdException(DbdException.gINVALID_STATEMENT_HANDLE);
             
         String objectName = methodName.substring(0, dot);
-        Debug.getLogWriter(Debug.TEDIOUS).println(mThreadId + 
-            "looking up object type " + objectName);
+        if (gLog.isTraceEnabled())
+            gLog.trace("Looking up object type " + objectName);
         Object object = null;
         if (objectName.equals("Statement") || 
             objectName.equals("PreparedStatement"))
@@ -1169,13 +1216,16 @@ class Connection implements Runnable
                 new String[] { objectName } );
         }
 
-        Debug.getLogWriter(Debug.TEDIOUS).println(mThreadId + 
-            "underlying object type: " + object.getClass().getName());
+        if (gLog.isDebugEnabled())
+        {
+            gLog.debug("Underlying object type: " + 
+                object.getClass().getName());
+        }
 
         String value = mHandleFunc(object, methodName.substring(dot + 1),
             aRequest.getParameters());
-        Debug.getLogWriter(Debug.VERBOSE).println(mThreadId +
-            methodName + " returned '" + value + "'");
+        if (gLog.isDebugEnabled())
+            gLog.debug(methodName + " returned '" + value + "'");
         try
         {
             return new StatementFuncResponse(value, 
@@ -1205,9 +1255,8 @@ class Connection implements Runnable
     private String mHandleFunc(Object anObject, String aMethodName,  
         Parameter[] aParameterList) throws SQLException,DbdException  
     { 
-        Debug.getLogWriter(Debug.TEDIOUS).println(mThreadId +  
-            "method has " + aParameterList.length + " parameters"); 
-
+        if (gLog.isDebugEnabled())
+            gLog.debug("Method has " + aParameterList.length + " parameters"); 
         Object[] parameterObjects = new Object[aParameterList.length];
         Class[] parameterClasses = new Class[aParameterList.length];
         try
@@ -1218,14 +1267,16 @@ class Connection implements Runnable
                 {
                     parameterObjects[i] = null;
                     parameterClasses[i] = mGetClass(aParameterList[i].type);
-                    Debug.getLogWriter(Debug.TEDIOUS).println(mThreadId + 
-                        "parameter " + (i + 1) + ": null; class " + 
-                        parameterClasses[i].getName());
+                    if (gLog.isTraceEnabled())
+                    {
+                        gLog.trace("Parameter " + (i + 1) + ": null; class " + 
+                            parameterClasses[i].getName());
+                    }
                     continue;
                 }
 
-                Debug.getLogWriter(Debug.TEDIOUS).print(mThreadId + 
-                    "parameter " + (i + 1) + ": ");
+                if (gLog.isTraceEnabled())                  
+                    gLog.trace("Parameter " + (i + 1) + ": ");
                 switch (aParameterList[i].type) 
                 {
                 case Types.BINARY:
@@ -1297,16 +1348,20 @@ class Connection implements Runnable
                     aParameterList[i].type == Types.VARBINARY ||
                     aParameterList[i].type == Types.LONGVARBINARY)
                 {
-                    Debug.getLogWriter(Debug.TEDIOUS).println(
-                        "byte[], length " + 
-                        ((byte[]) parameterObjects[i]).length +
-                        "; class " + parameterClasses[i].getName());
+                    if (gLog.isTraceEnabled())
+                    {
+                        gLog.trace("byte[], length " + 
+                            ((byte[]) parameterObjects[i]).length +
+                            "; class " + parameterClasses[i].getName());
+                    }
                 }
                 else
                 {
-                    Debug.getLogWriter(Debug.TEDIOUS).println(
-                        parameterObjects[i].toString() + "; class " + 
-                        parameterClasses[i].getName());
+                    if (gLog.isTraceEnabled())
+                    {
+                        gLog.trace(parameterObjects[i].toString() + 
+                            "; class " + parameterClasses[i].getName());
+                    }
                 }
             }
             
@@ -1323,9 +1378,12 @@ class Connection implements Runnable
                 int mod = c.getModifiers();
                 if (!Modifier.isPublic(mod))
                 {
-                    Debug.getLogWriter(Debug.TEDIOUS).println(mThreadId + 
-                        c.getName() + " is not public; looking for a " +
-                        "public interface with method " + aMethodName);
+                    if (gLog.isTraceEnabled())
+                    {
+                        gLog.trace(c.getName() + " is not public; looking " +
+                            "for a public interface with method " + 
+                            aMethodName);
+                    }
                     Class[] interfaces = c.getInterfaces(); 
                     for (int i = 0; i < interfaces.length; i++)
                     {
@@ -1336,10 +1394,12 @@ class Connection implements Runnable
                             {
                                 method = interfaces[i].getMethod(
                                     aMethodName, parameterClasses);
-                                Debug.getLogWriter(Debug.TEDIOUS).println(
-                                    mThreadId + "Found public interface " +
-                                    interfaces[i].getName() + 
-                                    " with method " + aMethodName); 
+                                if (gLog.isTraceEnabled())
+                                {
+                                    gLog.trace("Found public interface " +
+                                        interfaces[i].getName() + 
+                                        " with method " + aMethodName);
+                                }
                                 break;
                             }
                             catch (NoSuchMethodException e)
@@ -1352,8 +1412,8 @@ class Connection implements Runnable
                     }
                 }
                 
-                Debug.getLogWriter(Debug.TEDIOUS).println(mThreadId + 
-                    "found method: " + method.toString());
+                if (gLog.isTraceEnabled())
+                    gLog.trace("Found method: " + method.toString());
                 Object returnValue = null;
                 returnValue = method.invoke(anObject, parameterObjects ); 
                 // Null should be turned into BerNull. Boolean is
@@ -1521,7 +1581,7 @@ class Connection implements Runnable
      * @exception IOException if an error occurs reading from the stream
      */
     private byte[] mReadLong(int aColumnIndex, InputStream anInputStream, 
-        int aLongReadLen, boolean aLongTruncOk) 
+        int aLongReadLen, boolean aLongTruncOk, boolean readAll) 
         throws DbdException, IOException
     {
         if (aLongReadLen == 0)
@@ -1532,17 +1592,34 @@ class Connection implements Runnable
         byte[] buffer = new byte[sLONG_READ_BUFFER_SIZE];
         int read;
         int totalread = 0;
-        while (totalread < aLongReadLen &&
-            (read = anInputStream.read(buffer, 0, 
-            Math.min(buffer.length, (aLongReadLen - totalread)))) != -1)
+
+        if (readAll)
         {
-            totalread += read;
-            baos.write(buffer, 0, read);
+            while ((read = anInputStream.read(buffer, 0, buffer.length)) != -1)
+            {
+                totalread += read;
+                baos.write(buffer, 0, read);
+            }
+        }
+        else
+        {
+            while (totalread < aLongReadLen &&
+                (read = anInputStream.read(buffer, 0, 
+                    Math.min(buffer.length, (aLongReadLen - totalread)))) != -1)
+            {
+                totalread += read;
+                baos.write(buffer, 0, read);
+            }
         }
         baos.close();
-        Debug.getLogWriter(Debug.VERBOSE).println(mThreadId +
-            "read " + totalread + " bytes from LONG column " + aColumnIndex);
-        if (!aLongTruncOk && anInputStream.read() != -1)
+        if (gLog.isDebugEnabled())
+        {
+            gLog.debug("Read " + totalread + " bytes from LONG column " + 
+                aColumnIndex + "(LongReadLen=" + aLongReadLen + 
+                ";LongTruncOk=" + aLongTruncOk + ";jdbc_longreadall=" + 
+                readAll + ")");
+        }
+        if (!readAll && (!aLongTruncOk && anInputStream.read() != -1))
         {
             anInputStream.close();
             throw new DbdException(DbdException.gDATA_TRUNCATION);
@@ -1566,7 +1643,7 @@ class Connection implements Runnable
      * @exception IOException if an error occurs reading from the stream
      */
     private char[] mReadLong(int aColumnIndex, Reader aReader, 
-        int aLongReadLen, boolean aLongTruncOk) 
+        int aLongReadLen, boolean aLongTruncOk, boolean readAll) 
         throws DbdException, IOException
     {
         if (aLongReadLen == 0)
@@ -1577,17 +1654,34 @@ class Connection implements Runnable
         char[] buffer = new char[sLONG_READ_BUFFER_SIZE]; 
         int read;
         int totalread = 0;
-        while (totalread < aLongReadLen &&
-            (read = aReader.read(buffer, 0, 
-            Math.min(buffer.length, (aLongReadLen - totalread)))) != -1)
+
+        if (readAll)
         {
-            totalread += read;
-            out.write(buffer, 0, read);
+            while ((read = aReader.read(buffer, 0, buffer.length)) != -1)
+            {
+                totalread += read;
+                out.write(buffer, 0, read);
+            }
+        }
+        else
+        {
+            while (totalread < aLongReadLen &&
+                (read = aReader.read(buffer, 0, 
+                    Math.min(buffer.length, (aLongReadLen - totalread)))) != -1)
+            {
+                totalread += read;
+                out.write(buffer, 0, read);
+            }
         }
         out.close();
-        Debug.getLogWriter(Debug.VERBOSE).println(mThreadId +
-            "read " + totalread + " bytes from LONG column " + aColumnIndex);
-        if (!aLongTruncOk && aReader.read() != -1)
+        if (gLog.isDebugEnabled())
+        {
+            gLog.debug("Read " + totalread + " bytes from LONG column " + 
+                aColumnIndex + "(LongReadLen=" + aLongReadLen + 
+                ";LongTruncOk=" + aLongTruncOk + ";jdbc_longreadall=" +
+                readAll + ")");
+        }
+        if (!readAll && (!aLongTruncOk && aReader.read() != -1))
         {
             aReader.close();
             throw new DbdException(DbdException.gDATA_TRUNCATION);
@@ -1610,8 +1704,8 @@ class Connection implements Runnable
         {
             ErrorResponse error = new ErrorResponse(aSQLException,
                 mBerModule.getCharacterEncoding());
-            Debug.getLogWriter(Debug.TEDIOUS).println(mThreadId +
-                "sending error: " + aSQLException.getMessage());
+            if (gLog.isTraceEnabled())
+                gLog.trace("Sending error: " + aSQLException.getMessage());
             error.writeTo(mOut);
             mOut.flush();
         }
